"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const lsp = require("vscode-languageserver-protocol");
const symbols_1 = require("./symbols");
// Originally copied from https://github.com/theia-ide/typescript-language-server/blob/master/server/src/document-symbol.ts#L58
// togther with some util functions
var Position;
(function (Position) {
    function Min(...positions) {
        if (!positions.length) {
            return undefined;
        }
        let result = positions.pop();
        for (const p of positions) {
            if (isBefore(p, result)) {
                result = p;
            }
        }
        return result;
    }
    Position.Min = Min;
    function isBefore(one, other) {
        if (one.line < other.line) {
            return true;
        }
        if (other.line < one.line) {
            return false;
        }
        return one.character < other.character;
    }
    Position.isBefore = isBefore;
    function Max(...positions) {
        if (!positions.length) {
            return undefined;
        }
        let result = positions.pop();
        for (const p of positions) {
            if (isAfter(p, result)) {
                result = p;
            }
        }
        return result;
    }
    Position.Max = Max;
    function isAfter(one, other) {
        return !isBeforeOrEqual(one, other);
    }
    Position.isAfter = isAfter;
    function isBeforeOrEqual(one, other) {
        if (one.line < other.line) {
            return true;
        }
        if (other.line < one.line) {
            return false;
        }
        return one.character <= other.character;
    }
    Position.isBeforeOrEqual = isBeforeOrEqual;
})(Position || (Position = {}));
var Range;
(function (Range) {
    function intersection(one, other) {
        const start = Position.Max(other.start, one.start);
        const end = Position.Min(other.end, one.end);
        if (Position.isAfter(start, end)) {
            // this happens when there is no overlap:
            // |-----|
            //          |----|
            return undefined;
        }
        return lsp.Range.create(start, end);
    }
    Range.intersection = intersection;
})(Range || (Range = {}));
function asRange(sourceFile, textSpan) {
    return {
        start: ts.getLineAndCharacterOfPosition(sourceFile, textSpan.start),
        end: ts.getLineAndCharacterOfPosition(sourceFile, textSpan.start + textSpan.length),
    };
}
// Copy from typescript-langserver
function collectDocumentSymbols(sourceFile, parent, flatten = false) {
    const symbols = [];
    if (parent.childItems) {
        for (const item of parent.childItems) {
            collectDocumentSymbolsInRange(sourceFile, item, symbols, { start: asRange(sourceFile, item.spans[0]).start, end: asRange(sourceFile, item.spans[item.spans.length - 1]).end }, flatten);
        }
    }
    return symbols;
    // return collectDocumentSymbolsInRange(sourceFile, parent, symbols, { start: asRange(sourceFile, parent.spans[0]).start, end: asRange(sourceFile, parent.spans[parent.spans.length - 1]).end }, flatten);
}
exports.collectDocumentSymbols = collectDocumentSymbols;
// TODO clean up the logic here
function collectDocumentSymbolsInRange(sourceFile, parent, symbols, range, flatten = false) {
    // let shouldInclude = shouldIncludeEntry(parent);
    let shouldInclude = symbols_1.navigationTreeIsSymbol(parent);
    for (const span of parent.spans) {
        const spanRange = asRange(sourceFile, span);
        if (!Range.intersection(range, spanRange)) {
            continue;
        }
        const children = flatten ? symbols : [];
        if (parent.childItems) {
            for (const child of parent.childItems) {
                if (child.spans.some(childSpan => !!Range.intersection(spanRange, asRange(sourceFile, childSpan)))) {
                    const includedChild = collectDocumentSymbolsInRange(sourceFile, child, children, spanRange);
                    shouldInclude = shouldInclude || includedChild;
                }
            }
        }
        let selectionRange = spanRange;
        if (parent.nameSpan) {
            const nameRange = asRange(sourceFile, parent.nameSpan);
            // In the case of mergeable definitions, the nameSpan is only correct for the first definition.
            if (Range.intersection(spanRange, nameRange)) {
                selectionRange = nameRange;
            }
        }
        if (shouldInclude) {
            symbols.push({
                name: parent.text,
                detail: '',
                kind: symbols_1.stringtoSymbolKind(parent.kind),
                range: spanRange,
                selectionRange,
                children
            });
        }
        // TODO no need?
        break;
    }
    return shouldInclude;
}
//# sourceMappingURL=document-symbols.js.map