#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const opentracing_1 = require("opentracing");
const messages_1 = require("vscode-jsonrpc/lib/messages");
const connection_1 = require("./connection");
const lang_handler_1 = require("./lang-handler");
const logging_1 = require("./logging");
const typescript_service_1 = require("./typescript-service");
const packageJson = require('../package.json');
const program = require('commander');
const { initTracer } = require('jaeger-client');
program
    .version(packageJson.version)
    .option('-t, --trace', 'print all requests and responses')
    .option('-l, --logfile [file]', 'log to this file')
    .option('-j, --enable-jaeger', 'enable OpenTracing through Jaeger')
    .parse(process.argv);
const logger = program.logfile ? new logging_1.FileLogger(program.logfile) : new logging_1.StderrLogger();
const tracer = program.enableJaeger
    ? initTracer({ serviceName: 'javascript-typescript-langserver', sampler: { type: 'const', param: 1 } })
    : new opentracing_1.Tracer();
const options = {
    logMessages: program.trace,
    logger,
    tracer,
};
const messageEmitter = new connection_1.MessageEmitter(process.stdin, options);
const messageWriter = new connection_1.MessageWriter(process.stdout, options);
const remoteClient = new lang_handler_1.RemoteLanguageClient(messageEmitter, messageWriter);
const service = new typescript_service_1.TypeScriptService(remoteClient, options);
// Add an exit notification handler to kill the process
messageEmitter.on('message', message => {
    if (messages_1.isNotificationMessage(message) && message.method === 'exit') {
        logger.log(`Exit notification`);
        process.exit(0);
    }
});
connection_1.registerLanguageHandler(messageEmitter, messageWriter, service, options);
//# sourceMappingURL=language-server-stdio.js.map