"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
const iterare_1 = require("iterare");
const sinon = require("sinon");
const memfs_1 = require("../memfs");
const util_1 = require("../util");
chai.use(chaiAsPromised);
const assert = chai.assert;
describe('memfs.ts', () => {
    describe('InMemoryFileSystem', () => {
        describe('add()', () => {
            it('should add just a URI and emit an event', () => {
                const listener = sinon.spy();
                const fs = new memfs_1.InMemoryFileSystem('/');
                fs.on('add', listener);
                fs.add('file:///foo/bar.txt');
                sinon.assert.calledOnce(listener);
                sinon.assert.calledWithExactly(listener, 'file:///foo/bar.txt', undefined);
            });
            it('should add just a URI and emit an event when URI has encoded char', () => {
                const listener = sinon.spy();
                const fs = new memfs_1.InMemoryFileSystem('/');
                fs.on('add', listener);
                fs.add('file:///foo/%25bar.txt');
                sinon.assert.calledOnce(listener);
                sinon.assert.calledWithExactly(listener, 'file:///foo/%25bar.txt', undefined);
            });
            it('should add content for a URI and emit an event', () => {
                const listener = sinon.spy();
                const fs = new memfs_1.InMemoryFileSystem('/');
                fs.on('add', listener);
                fs.add('file:///foo/bar.txt', 'hello world');
                sinon.assert.calledOnce(listener);
                sinon.assert.calledWithExactly(listener, 'file:///foo/bar.txt', 'hello world');
            });
        });
        describe('uris()', () => {
            it('should hide TypeScript library files', () => __awaiter(this, void 0, void 0, function* () {
                const fs = new memfs_1.InMemoryFileSystem('/');
                assert.isFalse(iterare_1.default(fs.uris()).some(uri => memfs_1.typeScriptLibraries.has(util_1.uri2path(uri))));
            }));
        });
        describe('fileExists()', () => {
            it('should expose TypeScript library files', () => __awaiter(this, void 0, void 0, function* () {
                const fs = new memfs_1.InMemoryFileSystem('/');
                assert.isTrue(iterare_1.default(memfs_1.typeScriptLibraries.keys()).every(path => fs.fileExists(path)));
            }));
        });
        describe('readFile()', () => {
            it('should expose TypeScript library files', () => __awaiter(this, void 0, void 0, function* () {
                const fs = new memfs_1.InMemoryFileSystem('/');
                assert.isTrue(iterare_1.default(memfs_1.typeScriptLibraries.keys()).every(path => !!fs.readFile(path)));
            }));
        });
    });
});
//# sourceMappingURL=memfs.test.js.map